/***************************************************************************
                          controlprotocol.cpp  -  description
                             -------------------
    begin                : Mon Jul 29 2002
    copyright            : (C) 2002 by Willem Froehling
    email                : willem@koram.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "controlprotocol.h"
#include <iostream.h>
//#include "protocolmanager.h"

ControlProtocol::ControlProtocol(QString myID, SSL_Common* sslcom, SSL_CTX* ctx, QObject* parent=0) : QObject(parent)
{
        cout << "Building CP !!!" << endl;
        cp_myID = myID;
        cpsslcom = sslcom;
        cp_ctx = ctx;

        /* Soll das Server-Cert validiert werden */
        b_require_server_auth = TRUE;

        /* Statusflags, schner ist es diese in ein eigenes Objekt zu packen */
        b_connectionReq=FALSE;
        b_dataChannelReq=FALSE;
        b_dataReq = FALSE;
        b_roleUserIDRequest=FALSE;
        b_DataSent = FALSE;
        b_ScanReq = FALSE;
        b_authAnswerReq = FALSE;
        b_authResult = FALSE;

        data_totalBytes = 0;
}
ControlProtocol::~ControlProtocol()
{
        cout << "Destroying CP !!!" << endl;
}


int ControlProtocol::tcp_connect_control(const char* host,int port)
{
        cp_host = host;
        cp_port = port;
        struct hostent *hp;
        struct sockaddr_in addr;
        int sock, r, r2;

        if(!(hp=gethostbyname(cp_host)))
                cpsslcom->berr_exit("Kann Hostnamen nicht aufloesen");

        memset(&addr,0,sizeof(addr));
        addr.sin_addr=*(struct in_addr*) hp->h_addr_list[0];
        addr.sin_family=AF_INET;
        addr.sin_port=htons(cp_port);

        if((sock=socket(AF_INET,SOCK_STREAM, IPPROTO_TCP))<0)
                cpsslcom->err_exit("Kann Socket nicht erzeugen");

        if(::connect(sock,(struct sockaddr *)&addr, sizeof(addr))<0)
                cpsslcom->err_exit("Kann keine Verbindung herstellen");

        /* Connect the SSL socket */
        ssl=SSL_new(cp_ctx);

        if (ssl == NULL)
                cpsslcom->err_exit("Kann SSL-Objekt nicht erzeugen");

        sbio=BIO_new_socket(sock,BIO_NOCLOSE);
        SSL_set_bio(ssl,sbio,sbio);

        /* wir haben jetzt socket -> sbio -> ssl */

        r = SSL_connect(ssl);

        /*  Fehlerabfrage */
        r2 =  SSL_get_error(ssl, r);
        switch (r2) {
        case SSL_ERROR_NONE:
                displayData("   Kontrollkanal aufgebaut");
                break;
        case SSL_ERROR_ZERO_RETURN:
                cout << "zero error in CP SSL_connect" << endl;
                break;
        default:
                cout << "Irgendein Fehler in CP SSL_connect" << endl;
        }

        if(r <= 0) {
                cpsslcom->berr_exit("Irgendwas ging schief bei SSL_connect");
        }


        /* Serverzertifikat checken */
        if(b_require_server_auth)
                check_cert(ssl,cp_host);

        /* QSocketNotifier an socket binden */
        sn = new QSocketNotifier( sock, QSocketNotifier::Read, this );
        connect( sn, SIGNAL(activated(int)),this, SLOT(dataReceived()) );

        return(sock);
}


/* Common name testen mit dem hostnamem*/
void ControlProtocol::check_cert(SSL* ssl,const char* host)
{
        X509 *peer;
        char peer_CN[256];

        if(SSL_get_verify_result(ssl)!=X509_V_OK)
                cpsslcom->berr_exit("Serverzertifikat stimmt nicht");


        /*Check the common name*/
        peer=SSL_get_peer_certificate(ssl);
        X509_NAME_get_text_by_NID(X509_get_subject_name(peer), NID_commonName, peer_CN, 256);
        if(strcasecmp(peer_CN,host))
                cpsslcom->err_exit("Common name passt nicht zum Hostnamen");
}

/** Slot der getriggert wird, wenn der SocketNotifier zulesende Daten hat */
void ControlProtocol::dataReceived()
{
        int buf_len, index, index2;
        QString buffer, remote_data_hostname, remote_data_port;

        /* Daten lesen */
        buf_len = giveData(buffer);

        if (buffer.length() != (uint) buf_len) {
                buffer.truncate(buf_len);
        }

        displayData("..");

        if (buffer.contains("AuthConfirm+RoleAndUserIDRequest")) {
                displayData("< Auth_conf & Role+UserID_req");
                displayData("> Role+UserID");

                /* Verbindungsaufbau erfolgreich, */
                b_connectionReq=FALSE;
                b_roleUserIDRequest=TRUE;

                /* Hier kann man jetzt die Rolle und die UserID aus der GUI ablesen */
                emit getUserRole();
        }
        else if   (buffer.contains("UserIDConfirm")) {
                b_roleUserIDRequest=FALSE,
                                    b_dataChannelReq = TRUE;
                displayData("< Role+UserID confirm");
                displayData("> Data_Channel_req");
                sendToServer();
        }
        else if (buffer.contains("Data_Channel_Info")) {
                b_dataChannelReq = FALSE;
                b_dataChannelConf = TRUE;

                displayData("< Data_Channel_Info");
                /* Jetzt muss der Datenkanal aufgebaut werden */
                /* Hier wird der Host und Port aus der Serverantwort extrahiert */
                index = buffer.find("Host:");
                index2 = buffer.find("::");
                remote_data_hostname = buffer.mid(index+5, (index2-index-5));
                index = buffer.find("Port:");
                index2 = buffer.find(":::");
                remote_data_port = buffer.mid(index+5, (index2-index-5));
                displayData("  mit " + remote_data_hostname + ":" + remote_data_port);
                displayData("  baue Datenkanal auf");
                displayData("> Data_Channel_confirm");

                /* Datenkanal aufbauen */
                emit openDataChannel(remote_data_hostname, remote_data_port.toUInt());
        }
        else if (buffer.contains("ScanRequest+SecurityInfo")) {

                b_dataChannelConf = FALSE;
                b_ScanReq = TRUE;

                displayData("< ScanReq+SecInfo erhalten");

                index = buffer.find ("Methods:");
                /* index+8 muss gewhlt werden, da Methods: bersprungenwerden muss */
                index2 = buffer.find(":", index+8);

                if (index+8 != index2)
                        methods = buffer.mid( index+8, (index2-index-8) );
                else
                        methods = "NONE";

                displayData("  Methoden: " + methods);
                displayData("> Meta+Kalib.-Daten");
                sendToServer();

                /* Jetzt sollen die Aufzeichnungsdaten ber den Daten-Kanal bermittelt werden. */
                emit triggerSendBioData(methods);
        }
        else if (buffer.contains("Data_rcvd")) {
                b_DataSent = FALSE;
                b_authResult = TRUE;

                displayData("< Data_rcvd");
        }
        else if (buffer.contains("Auth_result")) {
                b_authResult = FALSE;
                //        b_authFin = TRUE;

                displayData("< Auth_Result");

                index = buffer.find(":");
                index2 = buffer.find(":", index+1);
                if (index+1 != index2) {
                        authResultStr = buffer.mid(index+1, (index2-index-1) );

                        if (authResultStr.contains("OK") )
                                displayData("  YES, Benutzer authentisiert :)");

                        else if (authResultStr.contains("NO") )
                                displayData("  NO, Benutzer **nicht** authentisiert :(");

                }
                else {
                        displayData("  Kein Auth-Ergebnis bermittelt");
                }
                //				sendToServer();
                displayData("******************************");
        }

        else {
                cpsslcom->err_exit("Lese Fehler, kein gltiges Protokoll! (CP)");
        }


}
/** Diese Methode gibt die empfangenen Daten in QString und die anzahl der Bytes zurck */
int ControlProtocol::giveData(QString& buf)
{
        int r, r2, len;
        char tmp_buffer[BUFSIZE];
        unsigned long error_num;
        char* buf2;
        len = 0;

        /* buf muss leer sein, sollte buff nicht leer sein, wird es geleert */
        if (!buf.isEmpty()) {
                buf.remove( 0, buf.length() );
        }

        /* Daten lesen */
        r=SSL_read(ssl,tmp_buffer,BUFSIZE);

        /* Fehler abfragen */
        switch(SSL_get_error(ssl,r)) {
        case SSL_ERROR_NONE:
                len=r;
                buf.append(tmp_buffer);
                break;

        case SSL_ERROR_ZERO_RETURN:
                r2=SSL_shutdown(ssl);

                switch(r2) {
                case 1:
                        //            cout << "ssl shutdown success" << endl;

                default:
                        cpsslcom->berr_exit("Shutdown failed");
                }

        case SSL_ERROR_SYSCALL:
                fprintf(stderr,"SSL Fehler: Premature close\n");
                error_num = ERR_get_error();
                buf2 = NULL;
                buf2 = ERR_error_string(error_num, buf2);

                SSL_free(ssl);

        default:
                cpsslcom->berr_exit("SSL :( Leseproblem");
        }

        return (len);
}


/* Je nach Zustand wurde ein Antwortmeldung zusammengebaut.
		Zum debuggen wird im BioClient auf einen Tastendruck gewartet
  	und anschliessend sendToServer2() aktiviert */
void ControlProtocol::sendToServer()
{
        emit checkStep();
}

/** SLOT fr das engltige Versenden nachdem step-debug berprft wurde */
void ControlProtocol::sendToServer2()
{
        QString buffer;

        buffer.append("ClientID:");
        buffer.append(cp_myID);
        buffer.append( ":\r\n");

        if (b_connectionReq) {
                /* Request-String bauen */
                buffer.append("AuthRequest\r\n");

                /* String senden */
                sendData(buffer);
                displayData("> AuthRequest gesendet");
        }
        else if ( b_roleUserIDRequest) {
                buffer.append("Role:");
                buffer.append(rolle);
                buffer.append("::UserID:");
                buffer.append(username);
                buffer.append("::\r\n");
                sendData(buffer);
        } else if ( b_dataChannelReq ) {
                buffer.append("DataChannelRequest\r\n");
                sendData(buffer);
        }
        else if (b_dataChannelConf) {
                buffer.append("Data_Channel_conf\r\n");
                sendData(buffer);
        }
        else if (b_ScanReq) {
                buffer.append("Meta+Kalib_Data::NOT_SPECIFIED_IN_THIS_EXAMPLE::\r\n");
                sendData(buffer);
        }
        else if (b_DataSent) {
                buffer.append("Data_sent:");
                buffer.append(QString::number(data_totalBytes));
                buffer.append(":\r\n");
                sendData(buffer);
                displayData("> Data_sent mit " + QString::number(data_totalBytes) + " Bytes");
        }
        else if (b_dataReq) {
                displayData("Empfangene Methoden: " + methods);
                buffer.append("AuthMethodACK\r\n");
                sendData(buffer);
        }
        else if (b_authFin) {
                cout << "Protokoll durchlaufen :)" << endl,
                buffer.append("NOOP\r\n");
                sendData(buffer);
                b_authFin = FALSE;
        }
        else {
                cout << "Fehler in sendToServer, nicht zu tun" << endl;
        }
}


/** Diese Methode sendet die Daten in QString an den Server und gibt die gesendeten Bytes zurck */
int ControlProtocol::sendData(QString& buf)
{
        int request_len;
        int r;

        request_len = buf.length();

        r=SSL_write(ssl,(const char*)buf,request_len);

        switch(SSL_get_error(ssl,r)) {
        case SSL_ERROR_NONE:
                if(request_len!=r)
                        cpsslcom->err_exit("SSL: Unvollstaendiges schreiben ! (CP)");
                break;
        default:
                cpsslcom->berr_exit("SSL  schreibproblem (CP)");
        }
        return(r);
}

/** Nachdem die aufgezeichneten biometrischen Daten geschickt wurden,lst
das Datenprotokoll dieses Signal ausschickt wurden  und teilt mit,  wieviele Bytes verschickt */
void ControlProtocol::bioDataSent(int tBytes)
{

        displayData("  schicke " + QString::number(tBytes) + " Bytes ber den Datenkanal");

        data_totalBytes = tBytes;
        if (b_ScanReq) {
                b_ScanReq = FALSE;
                b_DataSent = TRUE;
                sendToServer();
        }

}

/** Beendet den Kontrollkanal */
int ControlProtocol::closeConnection()
{
        int r,r2,  sock;

        /* SocketNotifier nicht mehr an den Socket verbinden */
        //    disconnect( sn, SIGNAL(activated(int)),this, SLOT(dataReceived()) );
        sock = sn->socket();

        r=SSL_shutdown(ssl);

        if(!r) {
                /* If we called SSL_shutdown() first then
                   we always get return value of '0'. In
                  this case, try again, but first send a
                   TCP FIN to trigger the other side's
                  close_notify*/
                r2 =shutdown(sock,SHUT_WR);

                r=SSL_shutdown(ssl);
                //    r2 = SSL_get_shutdown(ssl);
                //      cout << "SSL_get_shutdown 2. returm: " << r2 << endl;
                switch(SSL_get_error(ssl,r)) {
                case SSL_ERROR_NONE:

                        displayData("Verbindung erfolgreich abgebrochen");
                        break;
                case       SSL_ERROR_ZERO_RETURN:
                        cout <<"SSL_shutdown: zero_return" << endl;
                        break;
                case         SSL_ERROR_SYSCALL:
                        cout << "SSL_shutdown: syscall error " << endl;
                        break;
                case SSL_ERROR_WANT_READ:
                        cout << "SSL_shutdown: want read" << endl;
                        break;
                case SSL_ERROR_WANT_WRITE:
                        cout << "SSL_shutdown: want write " << endl;
                        break;
                default:
                        cout << "SSL_shutdown: sonstiger error" << endl;
                }

        }
        switch(r) {
        case 1:
                delete sn;
                SSL_free(ssl);
                r2=close(sock);
                r=0;
                break; /* Success */
        case 0:
                cout << "SSL_close_notify gesendet, obwohl 0" << endl;
                break; /* Success */

        case -1:
        default:
                cpsslcom->berr_exit("Shutdown CP Fehler");
        }

        return(r);
}
/** Hier wird der String fr den Benutzer und die Rolle verschickt */
void ControlProtocol::sendUserRole(QString usro_str)
{
        int index;

        index = usro_str.find("::");
        username = usro_str.mid(0, index);
        rolle = usro_str.mid(index+2, usro_str.length());

        displayData("  User: " + username + " und Rolle: " + rolle);
        sendToServer();
}

/** SLOT der die Variable b_sbs_pressed verndert,
wenn der Knopf in der Gui gedrckt wird. */
void ControlProtocol::sbs_pressed()
{
        cout << "debug step pressed" << endl;
}

