/***************************************************************************
                          controlprotocol.h  -  description
                             -------------------
    begin                : Mon Jul 29 2002
    copyright            : (C) 2002 by Willem Froehling
    email                : willem@koram.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTROLPROTOCOL_H
#define CONTROLPROTOCOL_H

#include "ssl_common.h"
#include <qsocketnotifier.h>


#define KEYFILE "client.pem"
#define PASSWORD "password"

/**
  *@author Willem Froehling
  */

class ControlProtocol : public QObject
{
        Q_OBJECT
public:
        ControlProtocol(QString myID, SSL_Common* sslcom, SSL_CTX* ctx, QObject* parent=0);
        ~ControlProtocol();

        /* SSL Verbindung zum Server herstellen */
        int tcp_connect_control(const char*, int);
        /* Daten ber den Kontrollkanal schicken */
        void sendToServer();
        /* Serverzertifikat prfen */
        void check_cert(SSL* ssl,const char* host);
        /** Beendet den Kontrollkanal */
        int closeConnection();

        /* globale Variable fr den Verbindungsaufbau */
        bool b_connectionReq;


signals:
        /** Wird vom ControlProtocol angetriggert, um die biometrischen Daten zu schicken */
        void triggerSendBioData(QString);

        /** SIGNAL um den Daten-Kanal zu ffnen und eine Verbindung mit dem Server aufzubauen */
        void openDataChannel(QString, Q_UINT16);
        /** Wird zum Abschluss eines Protokollsdurchlauf getriggert.
        Damit wird der Datenkanal geschlossen */
        void closeDataChannel();
        /** SIGNAL fr die Abfrage bei der Gui nach User und Role. */
        void getUserRole();
        /** SIGNAL zum Anzeigen von Text in der GUI */
        void displayData(QString);
        /** SIGNAL der vor dem Versenden ausgelst wird */
        void checkStep();

public slots:
        /** Slot der getriggert wird, wenn der SocketNotifier zu lesende Daten hat */
        void dataReceived();
        /** Das Dataprotokoll liefert hiermit die Anzahl der bertragenen Bytes */
        /** Nachdem die aufgezeichneten biometrischen Daten geschickt wurden,
        lst das Datenprotokoll dieses Signal aus */
        void bioDataSent(int tBytes);
        /** Hier wird der String fr den Benutzer und die Rolle verschickt */
        void sendUserRole(QString usro_str);
        /** SLOT der die Variable b_sbs_pressed verndert,
        wenn der Knopf in der Gui gedrckt wird. */
        void sbs_pressed();
        /** SLOT fr das engltige Versenden nachdem step-debug berprft wurde */
        void sendToServer2();

private:
        const char* cp_host;
        QString methods, cp_myID, authResultStr, rolle, username;
        int cp_port, data_totalBytes;
        SSL_Common* cpsslcom;
        QSocketNotifier* sn;
        SSL_CTX* cp_ctx;
        SSL* ssl;
        BIO* sbio;
        QObject* myparent;

        // Flags fr die Protokollschritte default=FALSE
        bool  b_dataChannelReq;
        bool  b_dataChannelConf;
        bool  b_ScanReq;
        bool b_dataReq;
        bool b_authAnswerReq;
        bool  b_roleUserIDRequest;
        bool b_require_server_auth;
        bool b_DataSent;
        bool b_authResult;
        /** Wenn das Ergebnis der Authentisierung erhalten wurde, wird hier der Status der Kommunikation gesetzt */
        bool b_authFin;

private:
        /** Diese Methode gibt die empfangenen Daten in QString und die anzahl der Bytes zurck */
        int giveData(QString& buf);
        /** Diese Methode sendet die Daten in QString an den Server und gibt die gesendeten Bytes zurck */
        int sendData(QString& buf);
};

#endif
