/***************************************************************************
                          dataprotocol.cpp  -  description
                             -------------------
    begin                : Wed Sep 18 2002
    copyright            : (C) 2002 by Willem Froehling
    email                : willem@koram.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dataprotocol.h"
#include <iostream.h>

DataProtocol::DataProtocol(QString myID, QString hostname, Q_UINT16 port, SSL_Common* sslcom, SSL_CTX* ctx, QObject* parent) : QObject(parent)
{
        cout << "Building DP !!!" << endl;
        struct hostent *hp;
        struct sockaddr_in addr;

        cout << "Create new DataProtocol " << endl;
        dpID = &myID;
        dpsslcom = sslcom;
        dp_ctx = ctx;

        /*Socket erzeugen und an ein SSL Objekt binden */

        if(!(hp=gethostbyname(hostname)))
                dpsslcom->berr_exit("Konnte den Hostnamen nicht auflsen (DP)");

        memset(&addr,0,sizeof(addr));
        addr.sin_addr=*(struct in_addr*) hp->h_addr_list[0];
        addr.sin_family=AF_INET;
        addr.sin_port=htons(port);

        if((sock=socket(AF_INET,SOCK_STREAM, IPPROTO_TCP))<0)
                dpsslcom->err_exit("Konnte Socket nicht erstellen (DP)");

        if(::connect(sock,(struct sockaddr *)&addr, sizeof(addr))<0)
                dpsslcom->err_exit("Konnte keine Verbindung herstellen (DP)");

        /* Connect the SSL socket */
        ssl=SSL_new(dp_ctx);
        sbio=BIO_new_socket(sock,BIO_NOCLOSE);
        SSL_set_bio(ssl,sbio,sbio);

        /* wir haben jetzt socket -> sbio -> ssl */

        if(SSL_connect(ssl)<=0)
                dpsslcom->berr_exit("SSL Verbindungsfehler (Datakanal");

}

DataProtocol::~DataProtocol()
{
        cout << "Destroying DP !!!" << endl;
}

/** Diese Methode sendet die Daten ber den TLS-Daten-Kanal */
int DataProtocol::sendData(QByteArray buf)
{
        int request_len;
        int r;

        request_len = buf.size();

        /* Daten schreiben */
        r=SSL_write(ssl,(const char*)buf,request_len);


        switch(SSL_get_error(ssl,r)) {
        case SSL_ERROR_NONE:
                if(request_len!=r)
                        dpsslcom->err_exit("Unvollstndiges schreiben! (DP)");
                break;
        default:
                dpsslcom->berr_exit("SSL Schreibproblem (DP)");
        }
        return(r);
}

/** Wird vom ControlProtocol getriggert, wenn
die Aufgezeichneten Daten ber den Daten
Kanal geschickt werden sollen */
void DataProtocol::triggerSendBioData(QString methoden)
{
        QString bildname = "bilder/Iris";
        QPixmap* transferBild;
        int sendbytes;

        /* Dateinamen zusammenbauen, je nachdem  welche Optionen in methods gesetzt wurden */
        if (methoden.contains("A")) {
                bildname.append("_lo");
        }

        if (methoden.contains("B")) {
                bildname.append("_ro");
        }

        if (methoden.contains("C")) {
                bildname.append("_lu");
        }

        if (methoden.contains("D")) {
                bildname.append("_ru");
        }

        bildname.append(".jpg");

        /* Das Bild laden */
        image = new FileLoader(bildname,this);

        /* Zeiger auf das geladene Bild erhalten (type: QPixmap)   */
        transferBild = image->getPixmap();

        QDataStream bufferStream(buffer, IO_WriteOnly);
        bufferStream << *transferBild;

        sendbytes = sendData(buffer);

        emit bioDataSent(sendbytes);
}

/** Beendet den Datenkanal */
int DataProtocol::closeConnection()
{
        int r,r2;

        r=SSL_shutdown(ssl);

        if(!r) {
                /* If we called SSL_shutdown() first then
                   we always get return value of '0'. In
                  this case, try again, but first send a
                   TCP FIN to trigger the other side's
                  close_notify*/
                r2 =shutdown(sock,SHUT_WR);

                r=SSL_shutdown(ssl);

                switch(SSL_get_error(ssl,r)) {
                case SSL_ERROR_NONE:
                        //        cout << "SSL_shutdown DP: none error" << endl;
                        break;
                case       SSL_ERROR_ZERO_RETURN:
                        cout <<"SSL_shutdown DP: zero_return" << endl;
                        break;
                case         SSL_ERROR_SYSCALL:
                        cout << "SSL_shutdown DP: syscall error " << endl;
                        break;
                case SSL_ERROR_WANT_READ:
                        cout << "SSL_shutdown DP: want read" << endl;
                        break;
                case SSL_ERROR_WANT_WRITE:
                        cout << "SSL_shutdown DP: want write " << endl;
                        break;
                default:
                        cout << "SSL_shutdown DP: sonstiger error" << endl;
                }

        }
        switch(r) {
        case 1:
                cout << "erfolgreich Datenkanal SSL Verbindung abgebaut" << endl;

                SSL_free(ssl);
                r2=close(sock);

                r=0;
                break; /* Success */
        case 0:
                cout << "SSL_close_notify gesendet, obwohl 0" << endl;
                break; /* Success */

                //    case -1:
        default:
                dpsslcom->berr_exit("Shutdown SSL DP fehlgeschlagen");
        }
        return(r);
}
