/***************************************************************************
                          protocolmanager.cpp  -  description
                             -------------------
    begin                : Mon Jul 1 2002
    copyright            : (C) 2002 by Willem Froehling
    email                : willem@koram.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "protocolmanager.h"

ProtocolManager::ProtocolManager(const QString& host, Q_UINT16& port, QObject* parent) : QObject(parent)
{
        hostname = host;
        portnum = port;
        myparent = parent;

        sock = 0;
        cp = NULL;
        dp = NULL;

        /* SSL context erzeugen */
        ssl_common = new SSL_Common(KEYFILE,PASSWORD);
        ctx = ssl_common->initialize_ctx();
        /*  random clientid seeden */
        srand((unsigned int)time((time_t *)NULL));

}

ProtocolManager::~ProtocolManager()
{
        cout << "Destroying PM !!!" << endl;
        ssl_common->destroy_ctx(ctx);
}

void ProtocolManager::startConnection()
{
        emit displayDataPM("************************");

        if (cp == NULL) {
                /* Hier wird die ID zufaellig erzeugt. Man knnte auch eine festvorgeben oder auslesen */

                myID = QString::number(::random());

                cp = new ControlProtocol(myID, ssl_common, ctx, this);

                connect( cp, SIGNAL(openDataChannel(QString, Q_UINT16)), this, SLOT(openDataChannel(QString, Q_UINT16)) );
                connect( cp, SIGNAL(closeDataChannel()), this, SLOT(closeDataChannel()) );

                connect( cp, SIGNAL(getUserRole()), myparent, SLOT(getUserRole()) );
                connect( myparent, SIGNAL(sendUserRole(QString)), cp, SLOT(sendUserRole(QString)) );

                connect( cp, SIGNAL(checkStep()), myparent, SLOT(checkStep()) );
                connect(myparent, SIGNAL(stepContinue()), cp, SLOT(sendToServer2()) );

                connect(cp, SIGNAL(displayData(QString)), this, SLOT(displayDataThrow(QString)) );
                displayDataPM("Neues Kontroll-Protokoll erzeugt");
        }

        if (sock == 0) {
                /* Verbindung aufbauen*/
                sock = cp->tcp_connect_control(hostname,portnum);
        } else
                emit  displayDataPM("System: Es existiert schon eine Verbindung! (PM)");

}

void ProtocolManager::closeConnection()
{
        int r = 0;

        if (dp != NULL) {
                r = dp->closeConnection();
                delete dp;
                dp = NULL;
        } else
                cout << "Fehler beim beenden DP: ist NULL " << r << endl;

        if (cp != NULL) {
                r = cp->closeConnection();
                delete cp;
                cp = NULL;

                if (r == 0) {
                        sock = 0;
                } else
                        cout << "Fehler beim beenden CP: " << r << endl;
        } else
                cout << "Fehler beim beenden CP: ist NULL" << endl;



}


void ProtocolManager::sendToServer()
{
        if ((cp != NULL) && (sock != 0)) {
                cp->b_connectionReq = TRUE;
                cp->sendToServer();
        } else
                displayDataPM("Es existiert noch keine Verbindung");
}

/** SLOT, wird vom ControlProtocol getriggert und erzeugt einen DatenKanal */
void ProtocolManager::openDataChannel( QString dp_hostname, Q_UINT16 dp_port)
{

        if (dp == NULL) {
                dp = new DataProtocol(myID, dp_hostname, dp_port, ssl_common, ctx, this);
                if (dp != NULL) {
                        connect( cp, SIGNAL(triggerSendBioData(QString)), dp, SLOT(triggerSendBioData(QString)) );
                        connect( dp, SIGNAL(bioDataSent(int)), cp , SLOT(bioDataSent(int)) );

                        /* Jetzt Data_Channel_confirm an Server commiten */
                        cp->sendToServer();
                } else
                        cout << "Fehler beim Datenkanal Aufbau !!! " << endl;

        } else {
                cout << "es existiert schon ein Datenkanal" << endl;
                cp->sendToServer();
        }
}
/** Slot zum beenden des Veerbindung des Datenkanals */
void ProtocolManager::closeDataChannel()
{
        int ret;
        disconnect( cp, SIGNAL(triggerSendBioData(QString)), dp, SLOT(triggerSendBioData(QString)) );
        disconnect( dp, SIGNAL(bioDataSent(int)), cp , SLOT(bioDataSent(int)) );
        ret = dp->closeConnection();
        delete dp;
        dp = NULL;
}
/** Leitet gui_str einfach weiter an die GUI */
void ProtocolManager::displayDataThrow(QString gui_str)
{
        emit displayDataPM(gui_str);
}
