/***************************************************************************
                          controlprotocol.cpp  -  description
                             -------------------
    begin                : Mon Aug 26 2002
    copyright            : (C) 2002 by Willem Froehling
    email                : willem@koram.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "controlprotocol.h"
#include <iostream.h>

ControlProtocol::ControlProtocol(int socket, SSL_Common* ssl_com_pm, SSL_CTX* ctx, QObject* parent=0) : QObject(parent)
{
        ssl_com_cp = ssl_com_pm;
        myctx = ctx;
        laufzahl = 0;
        /* BIO_new_socket() gibt eine buffered socket zurck */
        sbio=BIO_new_socket(socket,BIO_NOCLOSE);
        /* SSL_new - erzeugt eine neue SSL Struktur fr eine neue Verbindung */
        ssl_cp=SSL_new(myctx);
        /*SSL_set_bio - verbindet das SSL-Objekt mit einem BIO */
        SSL_set_bio(ssl_cp,sbio,sbio);
        /* Wir haben jetzt also:
        		socket -> sbio -> ssl_cp */

        /* Verbindung ber SSL annehmen
        SSL_accept gibt den status ber eine SSL_Verbindung zurck */
        cout << "Vor SSL_accept" << endl;
        if((r=SSL_accept(ssl_cp)<=0))
                ssl_com_cp->berr_exit("SSL accept error, no connection");

        /* Verbindung ist aufgebaut ...
        The BIO_new(BIO_f_buffer()) erzeugt ein neues buffered BIO*/
        io=BIO_new(BIO_f_buffer());
        /* BIO_f_ssl() returns the SSL BIO method. This is a filter BIO which is a
        					wrapper round the OpenSSL SSL routines adding a BIO "flavour" to SSL I/O */
        ssl_bio=BIO_new(BIO_f_ssl());
        /* BIO_set_ssl() sets the internal SSL pointer of BIO b to ssl using the close
            flag c. */
        BIO_set_ssl(ssl_bio,ssl_cp,BIO_CLOSE);
        /* The BIO_push(BIO *b,BIO *append) function appends the BIO append to b, it returns b.*/
        BIO_push(io,ssl_bio);

        /* wir haben jetzt :
        					socket->sbio->ssl->ssl_bio->io */

        sn = new QSocketNotifier( socket, QSocketNotifier::Read, this );
        connect( sn, SIGNAL(activated(int)),this, SLOT(dataReceived()) );

}

ControlProtocol::~ControlProtocol()
{
        cout << "Destroying CP-Object!!!!" << endl;
}

int ControlProtocol::giveData(QString& buf)
{
        int r,len;
        char tmp_buffer[BUFSIZE];

        len = 0;

        /* buf muss leer sein, sollte buff nicht leer sein, wird es geleert */
        if (!buf.isEmpty()) {
                buf.remove( 0, buf.length() );
        }

        /* geht beides BIO liest eine Zeile , SSL_read alles */
        r=BIO_gets(io,tmp_buffer,BUFSIZE-1);
        //r=SSL_read(ssl_cp, tmp_buffer,BUFSIZE-1);

        switch(SSL_get_error(ssl_cp,r)) {
        case SSL_ERROR_NONE:
                len=r;
                buf.append(tmp_buffer);
                break;

        case SSL_ERROR_ZERO_RETURN:
                cout << "CP: close_notify erhalten " << endl;
                disconnect( sn, SIGNAL(activated(int)),this, SLOT(dataReceived()) );
                closeConnection(sn->socket());

                len=0;
                break;

        default:
                ssl_com_cp->berr_exit("SSL read problem(giveData (CP1)");
        }

        return (len);
}

/** Slot der getriggert wird, wenn der SocketNotifier zulesende Daten hat */
void ControlProtocol::dataReceived()
{
        /* wir haben jetzt :
        					socket->sbio->ssl->ssl_bio->io */
        int buf_len, index, index2, client_sentBytes;

        if  (!sendBuffer.isEmpty())
                sendBuffer.remove( 0, sendBuffer.length() );

        buf_len = giveData(buffer);

        if (buf_len > 0) {
                if (buffer.contains("ClientID:")) {
                        index = buffer.find (":");
                        index2 = buffer.find(":", index+1);
                        if (index+1 != index2)
                                cp_clientID = buffer.mid( index+1, (index2-index-1) );

                        buf_len = giveData(buffer);
                }

                /* Ausgabe checken ob Verbindungsaufbau*/
                if (buffer.contains("AuthRequest")) {
                        emit displayData(cp_clientID.toInt(), "***********************************");
                        emit displayData(cp_clientID.toInt(), "< AuthRequest");
                        sendBuffer.append("AuthConfirm+RoleAndUserIDRequest\r\n");
                        buf_len = sendData(sendBuffer);

                }
                else if (buffer.contains("Role:")) {
                        index = buffer.find(":");
                        index2 = buffer.find("::", index+1);
                        if (index < index2)
                                authRole = buffer.mid(index+1, (index2-index-1));
                        index = buffer.find(":", index2+2) ;
                        index2 = buffer.find("::", index+1);
                        if (index < index2)
                                authUser = buffer.mid(index+1, (index2-index-1));
                        /* Hier msste jetzt die Abfrage rein, ob der user bekannt ist und welche methoden nutzbar sind */
                        /* In diesem Programm wird davon ausgegangen, das der User berechtigt ist */
                        emit displayData(cp_clientID.toInt(),"< Role+UserID mit");
                        emit displayData(cp_clientID.toInt(), "   Role: " + authRole + " und User: " + authUser);

                        sendBuffer.append("Role+UserIDConfirm\r\n");
                        buf_len = sendData(sendBuffer);

                }
                else if (buffer.contains("DataChannelRequest")) {
                        /* Datenkanal  Informationen zurck senden IP/PORT. */
                        /* eventuell vorher noch eine abfrage, ob  Datenkanal LISTEN */
                        emit displayData(cp_clientID.toInt(), "< Data_Channel_req");
                        /* signal DataChannelReq wird emmitiert */
                        emit dataChannelReq();

                }
                else if (buffer.contains("Data_Channel_conf")) {
                        emit displayData(cp_clientID.toInt(),"< Data_Channel_confirm");
                        /* Hier mssen jetzt die gewnschte SecInfo aus der Gui gelesen werden !!!*/
                        sendBuffer.append("ScanRequest+SecurityInfo ");
                        /* sendData wird diesmal nicht hier, sondern bei  cp_sendMethodList getriggert */
                        emit getMethods(cp_clientID);
                }
                else if (buffer.contains("Meta+Kalib_Data")) {
                        emit displayData(cp_clientID.toInt(),"< Meta+Kalib._Data");
                        emit displayData(cp_clientID.toInt(), "   empfange ber den Datenkanal");
                }
                else if (buffer.contains("Data_sent")) {
                        emit displayData(cp_clientID.toInt(),"< Data_sent");
                        index = buffer.find (":");
                        index2 = buffer.find(":", index+1);
                        if (index+1 != index2)
                                client_sentBytes = buffer.mid( index+1, (index2-index-1) ).toInt();

                        /* Die Antwort an den Client wird in der Methode check_bioData_result() geschickt */
                        emit check_bioData(client_sentBytes, cp_clientID);
                }
                else if (buffer.contains("NOOP")) {
                        emit displayData(cp_clientID.toInt(), "   Client hat NOOP geschickt :)");
                }
                else
                        ssl_com_cp->err_exit("Lese Fehler (dataRecieved (CP1))");
        }
}

/** Diese Methode sendet die Daten in QString an den client und gibt die Anzahl der gesendeten Bytes zurck */
int ControlProtocol::sendData(QString& s_buffer)
{
        emit displayData(cp_clientID.toInt(), "> " + s_buffer);
        if((r=BIO_puts(io,(const char*)s_buffer))<=0)
                ssl_com_cp->err_exit("Schreibe Fehler (CP)");
        if((r=BIO_flush(io))<0)
                ssl_com_cp->err_exit("Fehler: flushing BIO (CP)");

        return (r);
}

/** Empfngt die gewnschten Methode vom BioServer und sendet diese an den Client */
void ControlProtocol::cp_sendMethodList(QString mList, QString cID)
{
        int r;
        if (cID == cp_clientID) {
                sendBuffer.append("Methods:");
                sendBuffer.append(mList);
                sendBuffer.append(":\r\n");
                /* Abschicken */
                r = sendData(sendBuffer);
        }
}
/** Hier wird das Ergebnis der berprfung vom Datenkanal zurckgeschickt */
void ControlProtocol::check_bioData_result(bool d_result, QString cID)
{
        int buf_len;

        if (cID == cp_clientID) {
                if (d_result ) {

                        /* Sende Buffer leeren */
                        if  (!sendBuffer.isEmpty())
                                sendBuffer.remove( 0, sendBuffer.length() );

                        sendBuffer.append("Data_rcvd\r\n");
                        buf_len = sendData(sendBuffer);

                        emit getAuthStatus(cp_clientID);
                }
                else {
                        emit displayData(cp_clientID.toInt(),"Fehler bei der bertragung der Daten ber den Datenkanal :(");
                        /* Hier msste jetzt ein Rescan beim Client erbeten werden */
                }
        }
}

/** Das Ergbenis der Authentikation wird hier empfangen */
void ControlProtocol::sendAuthStatus(bool authResult, QString cID)
{
        int buf_len;
        if (cID == cp_clientID) {

                if  (!sendBuffer.isEmpty())
                        sendBuffer.remove( 0, sendBuffer.length() );

                sendBuffer.append("Auth_result:");

                if (authResult) {
                        sendBuffer.append("OK:\r\n");
                }
                else {
                        sendBuffer.append("NO:\r\n");
                }

                buf_len = sendData(sendBuffer);
        }
}


/** Dieser Slot bekommt den Port des Servers
fr den Datenkanal */
void ControlProtocol::dataChannelResult(int d_port)
{
        int buf_len;

        /* Portnummer zum String umwandeln */
        QString portNum;
        portNum = portNum.setNum(d_port);

        cout << "DATAPORT : " << d_port << endl;

        /* Antwortstring zusammenbauen */
        sendBuffer.append("Data_Channel_Info:+:K1:Host:127.0.0.1::Port:");
        sendBuffer.append(portNum);
        sendBuffer.append(":::\r\n");
        /* Antwort schicken */
        buf_len = sendData(sendBuffer);
}

/** Wenn der Kontrollkanal von der Gegenstelle ein SSL_close_notify erhlt, wird hier die Verbindung beendet */
void ControlProtocol::closeConnection(int sock)
{
        int r;

        r=SSL_shutdown(ssl_cp);
        switch(r) {
        case 1:
                close(sock);
                emit clientCut();

                break; /* Success */
        default:
                ssl_com_cp->berr_exit("Kontrollkanal SSL_Shutdown Fehler");
        }
}
