/***************************************************************************
                          controlprotocol.h  -  description
                             -------------------
    begin                : Mon Aug 26 2002
    copyright            : (C) 2002 by Willem Froehling
    email                : willem@koram.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONTROLPROTOCOL_H
#define CONTROLPROTOCOL_H

#include <qobject.h>
#include <qapplication.h>
#include "ssl_common.h"
#include <qsocketnotifier.h>
#include <qserversocket.h>

/**Diese Klasse kmmert sich um die Kommunikation mit einem auth-client
  *@author Willem Froehling
  */

class ControlProtocol : public QObject
{
        Q_OBJECT
public:
        ControlProtocol(int socket, SSL_Common* ssl_com_pm,SSL_CTX* ctx, QObject* parent=0);
        ~ControlProtocol();

signals:
        /** SIGNAL wenn ein neuer Client sich verbindet */
        void signal_newClientConnection();
        /** Dieses Signal triggert die Auswertung der Methodenliste an. */
        void getMethods(QString);
        /** Signal um im Protokollmanager einen Datenkanal zu ffnen */
        void dataChannelReq();
        /** Wenn der Client meldet, dass die bersendugn ber den Datenkanal abgeschlossen ist, kann hiermit berprft werden, ob alle Daten angekommen sind. */
        void check_bioData(int, QString);
        /** Hier wird die Checkbox als ersatz fr das Auth-Ergebnis abgefragt */
        void getAuthStatus(QString);
        /** SIGNAL um auf der GUI was anzuzeigen */
        void displayData(int, QString);
        /** Wenn die SSL-Verbindung vom Client beendet wird,
        dient dieses SIGNAL um den PM zu benachrichtigen
        */
        void clientCut();

public slots:
        /** Slot der getriggert wird, wenn der SocketNotifier zulesende Daten hat */
        void dataReceived();
        /** Empfngt die gewnschten Methode vom BioServer und sendet diese an den Client */
        void cp_sendMethodList(QString mList, QString);
        /** Das Ergbenis der Authentikation wird hier empfangen */
        void sendAuthStatus(bool, QString);
        /** Hier wird das Ergebnis der berprfung vom Datenkanal zurckgeschickt */
        void check_bioData_result(bool d_result, QString cID);
        /** Dieser Slot bekommt den Port des Servers
        fr den Datenkanal */
        void dataChannelResult(int d_port);
private:
        /** Methode zum Auslesen der empfangen Daten */
        int giveData(QString& buf);
        /** Diese Methode sendet die Daten in QString an den client und gibt die Anzahl der gesendeten Bytes zurck */
        int sendData(QString& s_buffer);
        /** Wenn der Kontrollkanal von der Gegenstelle ein SSL_close_notify erhlt, wird hier die Verbindung beendet */
        void closeConnection(int);

        SSL_Common* ssl_com_cp;
        SSL* ssl_cp;
        SSL_CTX* myctx;
        int    aktiv_socket, myID;
        BIO *sbio;
        int r,len;
        pid_t pid;
        BIO *io,*ssl_bio;
        QSocket* cp_socket;
        QSocketNotifier* sn;
        int laufzahl;
        QString buffer, sendBuffer, cp_clientID, authRole, authUser;

};

#endif
