/***************************************************************************
                          dataprotocol.cpp  -  description
                             -------------------
    begin                : Wed Sep 18 2002
    copyright            : (C) 2002 by Willem Froehling
    email                : willem@koram.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dataprotocol.h"
#include <iostream.h>

DataProtocol::DataProtocol(SSL_Common* ssl_com_pm, SSL_CTX* ctx, int d_port):QServerSocket(d_port)
{
        cout << "Buidling DP" << endl;
        listenPort = d_port;
        myctx = ctx;
        ssl_com_dp = ssl_com_pm;
        authBild = new Image();
}
DataProtocol::~DataProtocol()
{
        cout << "Destroy DP !!!" << endl;
        delete authBild;
        authBild = NULL;
}

void DataProtocol::newConnection(int d_socket)
{
        int r;
        cout << "new Datakanal Client Connection!" << endl;

        /* BIO_new_socket() gibt eine buffered socket zurck */
        sbio=BIO_new_socket(d_socket,BIO_NOCLOSE);

        /* SSL_new - erzeugt eine neue SSL Struktur fr eine neue Verbindung */
        ssl_dp=SSL_new(myctx);

        /*SSL_set_bio - verbindet das SSL-Objekt mit einem BIO */
        SSL_set_bio(ssl_dp,sbio,sbio);
        /* Wir haben jetzt also:
        		socket -> sbio -> ssl_cp */

        /* Verbindung ber SSL annehmen
        SSL_accept gibt den status ber eine SSL_Verbindung zurck */
        if((r=SSL_accept(ssl_dp)<=0))
                ssl_com_dp->berr_exit("SSL accept error, keine Verbindung (DATA-Channel)");
        else
                cout << "SSL-Verbindung hergestellt (DATA-Channel)"<< endl;

        /* Verbindung ist aufgebaut ...
        The BIO_new(BIO_f_buffer()) erzeugt ein neues buffered BIO*/
        io=BIO_new(BIO_f_buffer());
        /* BIO_f_ssl() returns the SSL BIO method. This is a filter BIO which is a
        					wrapper round the OpenSSL SSL routines adding a BIO "flavour" to SSL I/O */
        ssl_bio=BIO_new(BIO_f_ssl());
        /* BIO_set_ssl() sets the internal SSL pointer of BIO b to ssl using the close
            flag c. */
        BIO_set_ssl(ssl_bio,ssl_dp,BIO_CLOSE);

        /* The BIO_push(BIO *b,BIO *append) function appends the BIO append to b, it returns b.*/
        BIO_push(io,ssl_bio);

        /* wir haben jetzt :
        					socket->sbio->ssl->ssl_bio->io */

        /* Jetzt wird ein QSocketNotifier mit der Socket verbunden*/
        sn = new QSocketNotifier( d_socket, QSocketNotifier::Read);
        connect( sn, SIGNAL(activated(int)), this, SLOT(dataReceived()) );
}


/** Dieser SLOT wird aufgerufen, wenn Daten aus dem Netzwerk auftauchen */
void DataProtocol::dataReceived()
{

        int len,r;
        char tmp_buffer[BUFSIZE];
        len = 0;

        /* geht beides BIO liest eine Zeile , SSL_read alles */
        //r=BIO_gets(io,tmp_buffer,BUFSIZE-1);
        r=SSL_read(ssl_dp, tmp_buffer,BUFSIZE-1);

        switch(SSL_get_error(ssl_dp,r)) {
        case SSL_ERROR_NONE:
                authBild->appendData(tmp_buffer,r);
                break;

        case SSL_ERROR_ZERO_RETURN:
                cout << "SSL soll beendet werden!" << endl;
                closeConnection(sn->socket());
                len=0;
                break;

        default:
                ssl_com_dp->berr_exit("SSL read problem(giveData (DP))");
        }

        /*Auslesen der Anzahl von wartenden Bytes */
        len = SSL_pending(ssl_dp);

        while (len != 0) {
                /* geht beides BIO liest eine Zeile , SSL_read alles */
                //r=BIO_gets(io,tmp_buffer,BUFSIZE-1);
                r=SSL_read(ssl_dp, tmp_buffer,BUFSIZE-1);

                switch(SSL_get_error(ssl_dp,r)) {
                case SSL_ERROR_NONE:
                        authBild->appendData(tmp_buffer, r);
                        break;
                default:
                        ssl_com_dp->berr_exit("SSL read problem(giveData (DP))");
                }

                /*Auslesen der Anzahl von wartenden Bytes */
                len = SSL_pending(ssl_dp);
        }

}
/** Wird vom Kontrollprotokoll ausgelst, wenn der Client alles bertragen hat */
void DataProtocol::check_bioData(int cp_BytesRecv, QString cID)
{
        bool data_ok;

        displayData(cID.toInt(), "   Protokoll " + QString::number(cp_BytesRecv) + " Bytes");
        displayData(cID.toInt(), "   habe " + QString::number(authBild->giveBytes()) + " Bytes erhalten");

        if (cp_BytesRecv == authBild->giveBytes()) {
                authBild->showImage();
                data_ok = TRUE;
        }
        else {
                data_ok = FALSE;
        }

        /* Hier kann jetzt berprft werden, ob alle Daten angekommen sind ber den Datenkanal */
        emit     check_bioData_result(data_ok,cID);
}

void DataProtocol::dataChannelReq()
{
        emit dataChannelResult(listenPort);
}

/** Wenn der Kontrollkanal von der Gegenstelle ein SSL_close_notify erhlt, wird hier die Verbindung beendet */
void DataProtocol::closeConnection(int sock)
{
        int r;

        r=SSL_shutdown(ssl_dp);

        switch(r) {
        case 1:
                //	    r2 =close(sock);
                SSL_clear(ssl_dp);
                break; /* Success */
        case 0:
        case -1:
        default:
                ssl_com_dp->berr_exit("Shutdown fehlgeschagen");
        }
        emit dp_connectionClosed();
}
