/***************************************************************************
                          protocolmanager.cpp  -  description
                             -------------------
    begin                : Tue Aug 20 2002
    copyright            : (C) 2002 by Willem Froehling
    email                : willem@koram.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "protocolmanager.h"
#include <iostream.h>

ProtocolManager::ProtocolManager(const QString& host, Q_UINT16& port, QObject* parent=0) : QServerSocket(port,0, parent)
{
        cout << "Server lauscht auf: " << host << ":" << port << endl;
        myparent = parent;
        cp = NULL;
        dp = NULL;
        dataport = 4343;
        /* Build our SSL context*/
        ssl_com_pm = new SSL_Common(KEYFILE,PASSWORD);
        ctx = ssl_com_pm->initialize_ctx();
        ssl_com_pm->load_dh_params(ctx,DHFILE);
}

ProtocolManager::~ProtocolManager()
{
        cout << "Destroying PM !!!!" << endl;
}

/* Wenn ein neuer Client sich mit dem Server verbindet, wird eine neue Instanz vom ControlProtocol
    gebildet */
void ProtocolManager::newConnection(int socket)
{

        emit displayData(0,"Neue Verbindung (CP)");
        cp = new ControlProtocol(socket, ssl_com_pm, ctx, (QObject*) this);

        dp = new DataProtocol(ssl_com_pm, ctx, (Q_UINT16)dataport);

        connect( cp, SIGNAL(getMethods(QString)), myparent, SLOT(getCheckBoxStatus(QString)) );
        connect( myparent, SIGNAL(sendCheckBoxStatus(QString, QString)), cp, SLOT(cp_sendMethodList(QString, QString)) );

        connect( cp, SIGNAL(getAuthStatus(QString)), myparent, SLOT(getAuthStatus(QString)) );
        connect( myparent, SIGNAL(sendAuthStatus(bool, QString)), cp, SLOT(sendAuthStatus(bool, QString)) );

        connect(cp, SIGNAL(dataChannelReq()), dp, SLOT(dataChannelReq()) );
        connect( dp, SIGNAL(dataChannelResult(int)), cp , SLOT(dataChannelResult(int)) );




        connect( cp, SIGNAL(check_bioData(int, QString)), dp, SLOT(check_bioData(int, QString)) );
        connect( dp, SIGNAL(check_bioData_result(bool, QString)), cp, SLOT(check_bioData_result(bool, QString)) );

        connect( cp, SIGNAL(displayData(int,QString)), this, SLOT(displayDataPassthrow(int, QString)) );
        connect( dp, SIGNAL(displayData(int,QString)), this, SLOT(displayDataPassthrow(int, QString)) );

        connect( cp, SIGNAL(clientCut()), this, SLOT(clientCut()) );
        connect( dp, SIGNAL(dp_connectionClosed()), this, SLOT(dp_connectionClosed()) );

        dataport = dataport + 1;

}


/** SLOT der einfach weiterreicht zu der GUI */
void ProtocolManager::displayDataPassthrow(int cID,QString cString)
{
        emit displayData(cID,cString);
}

/** Slot der getriggert wird, wenn die Netzverbindung beendet wird */
void ProtocolManager::clientCut()
{
        displayData(0,"Verbindung beendet");
        delete cp;
        cp = NULL;

        //Server strzt ab
        //   dp->closeConnection(dp->listenPort);
        delete dp;
        dp = NULL;
}

/** SLOT der benutzt wird, wenn der Datenkanal eine Verbindung beendet */
void ProtocolManager::dp_connectionClosed()
{
        displayData(0,"Datenkanal abgebaut");

        delete dp;
        dp = NULL;
}

