/***************************************************************************
                          ssl_common.cpp  -  description
                             -------------------
    begin                : Tue Aug 20 2002
    copyright            : (C) 2002 by Willem Froehling
    email                : willem@koram.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ssl_common.h"
#include <iostream.h>

SSL_Common::SSL_Common(char* keyfile, char* password)
{
        bio_err = NULL;
        pass4cb = password;
        SSL_Common::keyfile = keyfile;
}

SSL_Common::~SSL_Common()
{}

/* Simple Fehlerroutine */
int SSL_Common::err_exit(char* string)
{
        fprintf(stderr,"%s\n",string);
        exit(0);
}

/* SSL Fehler ausgeben und beenden */
int SSL_Common::berr_exit(char* string)
{
        if (bio_err == NULL) {
                exit(0);
        } else {
                BIO_printf(bio_err,"%s\n",string);
                ERR_print_errors(bio_err);
                exit(0);
        }
}

void SSL_Common::sigpipe_handle(int x)
{}

SSL_CTX* SSL_Common::initialize_ctx()
{
        SSL_METHOD* meth;
        SSL_CTX* ctx;
        if(bio_err == NULL) {
                /* Global system initialization*/
                SSL_library_init();
                SSL_load_error_strings();

                /* Fehlerausgabe binden */
                bio_err=BIO_new_fp(stderr,BIO_NOCLOSE);
                cout << "init SSL!" << endl;
        }

        /* SIGPIPE handler aufsetzen */
        signal(SIGPIPE,sigpipe_handle);

        /* Eigenen Context erstellen */
        meth=SSLv23_method();
        ctx=SSL_CTX_new(meth);

        if (ctx == NULL) {
                berr_exit("Kein CTX Object");
                return (NULL);
        } else {
                /* Zertifikate und Schlssel laden */
                if(!(SSL_CTX_use_certificate_chain_file(ctx,
                                                        keyfile)))
                        berr_exit("Kann Zertifikatdatei nicht lesen");

                SSL_CTX_set_default_passwd_cb(ctx,
                                              password_cb);

                if(!(SSL_CTX_use_PrivateKey_file(ctx,
                                                 keyfile,SSL_FILETYPE_PEM)))
                        berr_exit("Kann Schlsseldatei nicht lesen");

                /* Load the CAs we trust*/
                if(!(SSL_CTX_load_verify_locations(ctx,
                                                   CA_LIST,0)))
                        berr_exit("Kann CA-Datei nicht lesen");

#if (OPENSSL_VERSION_NUMBER < 0x00905100L)

                SSL_CTX_set_verify_depth(ctx,1);
#endif

                return(ctx);
        }
}

void SSL_Common::load_dh_params(SSL_CTX* ctx,char* file)
{
        DH *ret=0;
        BIO *bio;

        if ((bio=BIO_new_file(file,"r")) == NULL)
                berr_exit("Kann DH Datei nicht oeffnen");

        ret=PEM_read_bio_DHparams(bio,NULL,NULL,
                                  NULL);
        BIO_free(bio);
        if(SSL_CTX_set_tmp_dh(ctx,ret)<0)
                berr_exit("Konnte DH-Parameter nicht setzen");
}

void SSL_Common::generate_eph_rsa_key(SSL_CTX* ctx)
{
        RSA *rsa;

        rsa=RSA_generate_key(512,RSA_F4,NULL,NULL);

        if (!SSL_CTX_set_tmp_rsa(ctx,rsa))
                berr_exit("Konnte RSA Parameter nicht setzen");

        RSA_free(rsa);
}

void SSL_Common::destroy_ctx(SSL_CTX* ctx)
{
        SSL_CTX_free(ctx);
}



